@@ -8,6 +8,36 @@
 
 using namespace Rcpp;
 
+class RLASExtrabyteAttributes
+{
+  public:
+    RLASExtrabyteAttributes();
+    int id;
+    int start;
+    std::string name;
+    int type;
+    bool has_scale;
+    bool has_offset;
+    bool has_no_data;
+    bool has_min;
+    bool has_max;
+    double scale;
+    double offset;
+    double no_data;
+    double min;
+    double max;
+    std::vector<int> eb32;
+    std::vector<double> eb64;
+
+    bool is_supported();
+    bool is_32bits();
+    void push_back(LASpoint*);
+
+  private:
+    F64 get_attribute_double(LASpoint*);
+    I32 get_attribute_int(LASpoint*);
+};
+
 class RLASstreamer
 {
   public:
@@ -37,8 +67,6 @@ class RLASstreamer
     void read_rgb(bool);
     void read_nir(bool);
     void read_eb(IntegerVector); // extra byte numbers
-    F64 get_attribute_double(LASpoint*, I32);
-    I32 get_attribute_int(LASpoint*, I32);
 
   private:
     void initialize_bool();
@@ -92,9 +120,8 @@ class RLASstreamer
     bool p;
     bool rgb;
     bool nir;
-    std::vector<int> eb; // extra_byte attribute numbers
-    std::vector<int> eb32;
-    std::vector<int> eb64;
+    std::vector<RLASExtrabyteAttributes> extra_bytes_attr;
+    std::vector<int> eb;
 };
 
 #endif //LASSTREAMER_H
