@@ -2,11 +2,11 @@
 ===============================================================================
 
   FILE:  lasreader_asc.cpp
-
+  
   CONTENTS:
-
+  
     see corresponding header file
-
+  
   PROGRAMMERS:
 
     martin.isenburg@rapidlasso.com  -  http://rapidlasso.com
@@ -21,20 +21,17 @@
 
     This software is distributed WITHOUT ANY WARRANTY and without even the
     implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-
+  
   CHANGE HISTORY:
-
-    20 December 2016 -- by Jean-Romain Roussel -- Change fprint(stderr, ...), raise an exeption
-
+  
     see corresponding header file
-
+  
 ===============================================================================
 */
 #include "lasreader_asc.hpp"
 
 #include <stdlib.h>
 #include <string.h>
-#include <stdexcept>
 
 #ifdef _WIN32
 #include <windows.h>
@@ -46,7 +43,7 @@ BOOL LASreaderASC::open(const CHAR* file_name, BOOL comma_not_point)
 {
   if (file_name == 0)
   {
-    throw std::runtime_error(std::string("ERROR: fine name pointer is zero"));
+    REprintf("ERROR: file name pointer is zero\n");
     return FALSE;
   }
 
@@ -56,13 +53,13 @@ BOOL LASreaderASC::open(const CHAR* file_name, BOOL comma_not_point)
   file = fopen_compressed(file_name, "r", &piped);
   if (file == 0)
   {
-    throw std::runtime_error(std::string("ERROR: cannot open file '%s'")); //file_name
+    REprintf( "ERROR: cannot open file '%s'\n", file_name);
     return FALSE;
   }
 
   if (setvbuf(file, NULL, _IOFBF, 10*LAS_TOOLS_IO_IBUFFER_SIZE) != 0)
   {
-    throw std::runtime_error(std::string("WARNING: setvbuf() failed with buffer size %d")); //10*LAS_TOOLS_IO_IBUFFER_SIZE
+    REprintf( "WARNING: setvbuf() failed with buffer size %d\n", 10*LAS_TOOLS_IO_IBUFFER_SIZE);
   }
 
   // clean the header
@@ -216,7 +213,7 @@ BOOL LASreaderASC::open(const CHAR* file_name, BOOL comma_not_point)
 
   if (!complete)
   {
-    throw std::runtime_error(std::string("ERROR: was not able to find header"));
+    REprintf("ERROR: was not able to find header\n");
     return FALSE;
   }
 
@@ -255,9 +252,9 @@ BOOL LASreaderASC::open(const CHAR* file_name, BOOL comma_not_point)
         if (!fgets(line, line_size, file))
         {
 #ifdef _WIN32
-          throw std::runtime_error(std::string("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %I64d points")); //row, nrows, col, ncols, p_count
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %I64d points\n", row, nrows, col, ncols, p_count);
 #else
-          throw std::runtime_error(std::string("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %lld points")); //row, nrows, col, ncols, p_count
+          REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %lld points\n", row, nrows, col, ncols, p_count);
 #endif
         }
 
@@ -312,7 +309,7 @@ BOOL LASreaderASC::open(const CHAR* file_name, BOOL comma_not_point)
   }
   else
   {
-    throw std::runtime_error(std::string("WARNING: ASC raster contains only no data values"));
+    REprintf("WARNING: ASC raster contains only no data values\n");
     header.min_z = 0;
     header.max_z = 0;
   }
@@ -369,9 +366,9 @@ BOOL LASreaderASC::read_point_default()
       if (!fgets(line, line_size, file))
       {
 #ifdef _WIN32
-        throw std::runtime_error(std::string("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %I64d points")); //row, nrows, col, ncols, p_count
+        REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %I64d points\n", row, nrows, col, ncols, p_count);
 #else
-        throw std::runtime_error(std::string("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %lld points")); //row, nrows, col, ncols, p_count
+        REprintf("WARNING: end-of-file after %d of %d rows and %d of %d cols. read %lld points\n", row, nrows, col, ncols, p_count);
 #endif
         npoints = p_count;
         return FALSE;
@@ -440,20 +437,20 @@ BOOL LASreaderASC::reopen(const CHAR* file_name)
 {
   if (file_name == 0)
   {
-    throw std::runtime_error(std::string("ERROR: fine name pointer is zero"));
+    REprintf("ERROR: file name pointer is zero\n");
     return FALSE;
   }
 
   file = fopen_compressed(file_name, "r", &piped);
   if (file == 0)
   {
-    throw std::runtime_error(std::string("ERROR: cannot reopen file '%s'")); //file_name
+    REprintf( "ERROR: cannot reopen file '%s'\n", file_name);
     return FALSE;
   }
 
   if (setvbuf(file, NULL, _IOFBF, 10*LAS_TOOLS_IO_IBUFFER_SIZE) != 0)
   {
-    throw std::runtime_error(std::string("WARNING: setvbuf() failed with buffer size %d")); //10*LAS_TOOLS_IO_IBUFFER_SIZE
+    REprintf( "WARNING: setvbuf() failed with buffer size %d\n", 10*LAS_TOOLS_IO_IBUFFER_SIZE);
   }
 
   // read the header lines
@@ -601,8 +598,8 @@ void LASreaderASC::populate_bounding_box()
 
   if ((header.min_x > 0) != (dequant_min_x > 0))
   {
-    throw std::runtime_error(std::string("WARNING: quantization sign flip for min_x from %g to %g.")); //header.min_x, dequant_min_x
-    throw std::runtime_error(std::string("         set scale factor for x coarser than %g with '-rescale'")); //header.x_scale_factor
+    REprintf( "WARNING: quantization sign flip for min_x from %g to %g.\n", header.min_x, dequant_min_x);
+    REprintf( "         set scale factor for x coarser than %g with '-rescale'\n", header.x_scale_factor);
   }
   else
   {
@@ -610,8 +607,8 @@ void LASreaderASC::populate_bounding_box()
   }
   if ((header.max_x > 0) != (dequant_max_x > 0))
   {
-    throw std::runtime_error(std::string("WARNING: quantization sign flip for max_x from %g to %g.")); //header.max_x, dequant_max_x
-    throw std::runtime_error(std::string("         set scale factor for x coarser than %g with '-rescale'")); //header.x_scale_factor
+    REprintf( "WARNING: quantization sign flip for max_x from %g to %g.\n", header.max_x, dequant_max_x);
+    REprintf( "         set scale factor for x coarser than %g with '-rescale'\n", header.x_scale_factor);
   }
   else
   {
@@ -619,8 +616,8 @@ void LASreaderASC::populate_bounding_box()
   }
   if ((header.min_y > 0) != (dequant_min_y > 0))
   {
-    throw std::runtime_error(std::string("WARNING: quantization sign flip for min_y from %g to %g.")); //header.min_y, dequant_min_y
-    throw std::runtime_error(std::string("         set scale factor for y coarser than %g with '-rescale'")); //header.y_scale_factor
+    REprintf( "WARNING: quantization sign flip for min_y from %g to %g.\n", header.min_y, dequant_min_y);
+    REprintf( "         set scale factor for y coarser than %g with '-rescale'\n", header.y_scale_factor);
   }
   else
   {
@@ -628,8 +625,8 @@ void LASreaderASC::populate_bounding_box()
   }
   if ((header.max_y > 0) != (dequant_max_y > 0))
   {
-    throw std::runtime_error(std::string("WARNING: quantization sign flip for max_y from %g to %g.")); //header.max_y, dequant_max_y
-    throw std::runtime_error(std::string("         set scale factor for y coarser than %g with '-rescale'")); //header.y_scale_factor
+    REprintf( "WARNING: quantization sign flip for max_y from %g to %g.\n", header.max_y, dequant_max_y);
+    REprintf( "         set scale factor for y coarser than %g with '-rescale'\n", header.y_scale_factor);
   }
   else
   {
@@ -637,8 +634,8 @@ void LASreaderASC::populate_bounding_box()
   }
   if ((header.min_z > 0) != (dequant_min_z > 0))
   {
-    throw std::runtime_error(std::string("WARNING: quantization sign flip for min_z from %g to %g.")); //header.min_z, dequant_min_z
-    throw std::runtime_error(std::string("         set scale factor for z coarser than %g with '-rescale'")); //header.z_scale_factor
+    REprintf( "WARNING: quantization sign flip for min_z from %g to %g.\n", header.min_z, dequant_min_z);
+    REprintf( "         set scale factor for z coarser than %g with '-rescale'\n", header.z_scale_factor);
   }
   else
   {
@@ -646,8 +643,8 @@ void LASreaderASC::populate_bounding_box()
   }
   if ((header.max_z > 0) != (dequant_max_z > 0))
   {
-    throw std::runtime_error(std::string("WARNING: quantization sign flip for max_z from %g to %g.")); //header.max_z, dequant_max_z
-    throw std::runtime_error(std::string("         set scale factor for z coarser than %g with '-rescale'")); //header.z_scale_factor
+    REprintf( "WARNING: quantization sign flip for max_z from %g to %g.\n", header.max_z, dequant_max_z);
+    REprintf( "         set scale factor for z coarser than %g with '-rescale'\n", header.z_scale_factor);
   }
   else
   {
