@@ -2,11 +2,11 @@
 ===============================================================================
 
   FILE:  laswritercompatible.cpp
-
+  
   CONTENTS:
-
+  
     see corresponding header file
-
+  
   PROGRAMMERS:
 
     martin.isenburg@rapidlasso.com  -  http://rapidlasso.com
@@ -21,20 +21,17 @@
 
     This software is distributed WITHOUT ANY WARRANTY and without even the
     implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-
+  
   CHANGE HISTORY:
-
-    20 December 2016 -- by Jean-Romain Roussel -- Change fprint(stderr, ...), raise an exeption
-
+  
     see corresponding header file
-
+  
 ===============================================================================
 */
 #include "laswritercompatible.hpp"
 
 #include "bytestreamout_array.hpp"
 #include "bytestreamin_array.hpp"
-#include <stdexcept>
 
 BOOL LASwriterCompatibleDown::open(LASheader* header, LASwriteOpener* laswriteopener, BOOL moveCRSfromEVLRtoVLR, BOOL moveEVLRtoVLR)
 {
@@ -60,6 +57,10 @@ BOOL LASwriterCompatibleDown::open(LASheader* header, LASwriteOpener* laswriteop
   {
     return FALSE;
   }
+  if (header->extended_number_of_point_records > U32_MAX) // only less than 2^32-1 points are supported
+  {
+    return FALSE;
+  }
   this->header = header;
 
   // downgrade it to LAS 1.2 or LAS 1.3
@@ -93,8 +94,8 @@ BOOL LASwriterCompatibleDown::open(LASheader* header, LASwriteOpener* laswriteop
   else
     out = new ByteStreamOutArrayBE();
   // write control info
-  U16 lastools_version = (U16)LAS_TOOLS_VERSION;
-  out->put16bitsLE((U8*)&lastools_version);
+  U16 laszip_version = (U16)LASZIP_VERSION_BUILD_DATE;
+  out->put16bitsLE((U8*)&laszip_version);
   U16 compatible_version = 3;
   out->put16bitsLE((U8*)&compatible_version);
   U32 unused = 0;
@@ -104,9 +105,9 @@ BOOL LASwriterCompatibleDown::open(LASheader* header, LASwriteOpener* laswriteop
   if (start_of_waveform_data_packet_record != 0)
   {
 #ifdef _WIN32
-    throw std::runtime_error(std::string("WARNING: header->start_of_waveform_data_packet_record is %I64d. writing 0 instead.")); //start_of_waveform_data_packet_record
+    REprintf("WARNING: header->start_of_waveform_data_packet_record is %I64d. writing 0 instead.\n", start_of_waveform_data_packet_record);
 #else
-    throw std::runtime_error(std::string("WARNING: header->start_of_waveform_data_packet_record is %lld. writing 0 instead.")); //start_of_waveform_data_packet_record
+    REprintf("WARNING: header->start_of_waveform_data_packet_record is %lld. writing 0 instead.\n", start_of_waveform_data_packet_record);
 #endif
     start_of_waveform_data_packet_record = 0;
   }
@@ -115,9 +116,9 @@ BOOL LASwriterCompatibleDown::open(LASheader* header, LASwriteOpener* laswriteop
   if (start_of_first_extended_variable_length_record != 0)
   {
 #ifdef _WIN32
-    throw std::runtime_error(std::string("WARNING: EVLRs not supported. header->start_of_first_extended_variable_length_record is %I64d. writing 0 instead.")); //start_of_first_extended_variable_length_record
+    REprintf("WARNING: EVLRs not supported. header->start_of_first_extended_variable_length_record is %I64d. writing 0 instead.\n", start_of_first_extended_variable_length_record);
 #else
-    throw std::runtime_error(std::string("WARNING: EVLRs not supported. header->start_of_first_extended_variable_length_record is %lld. writing 0 instead.")); //start_of_first_extended_variable_length_record
+    REprintf("WARNING: EVLRs not supported. header->start_of_first_extended_variable_length_record is %lld. writing 0 instead.\n", start_of_first_extended_variable_length_record);
 #endif
     start_of_first_extended_variable_length_record = 0;
   }
@@ -125,23 +126,42 @@ BOOL LASwriterCompatibleDown::open(LASheader* header, LASwriteOpener* laswriteop
   U32 number_of_extended_variable_length_records = header->number_of_extended_variable_length_records;
   if (number_of_extended_variable_length_records != 0)
   {
-    throw std::runtime_error(std::string("WARNING: EVLRs not supported. header->number_of_extended_variable_length_records is %u. writing 0 instead.")); //number_of_extended_variable_length_records
+    REprintf("WARNING: EVLRs not supported. header->number_of_extended_variable_length_records is %u. writing 0 instead.\n", number_of_extended_variable_length_records);
     number_of_extended_variable_length_records = 0;
   }
   out->put32bitsLE((U8*)&number_of_extended_variable_length_records);
   U64 extended_number_of_point_records;
   if (header->number_of_point_records)
+  {
     extended_number_of_point_records = header->number_of_point_records;
+    REprintf("WARNING: legacy number_of_point_records in header of LAS 1.4 file should be zero.\n");
+  }
   else
+  {
     extended_number_of_point_records = header->extended_number_of_point_records;
+    header->number_of_point_records = (U32)header->extended_number_of_point_records;
+  }
   out->put64bitsLE((U8*)&extended_number_of_point_records);
   U64 extended_number_of_points_by_return;
   for (i = 0; i < 15; i++)
   {
-    if ((i < 5) && header->number_of_points_by_return[i])
-      extended_number_of_points_by_return = header->number_of_points_by_return[i];
+    if (i < 5)
+    {
+      if (header->number_of_points_by_return[i])
+      {
+        extended_number_of_points_by_return = header->number_of_points_by_return[i];
+        REprintf("WARNING: legacy number_of_points_by_return[%d] in header of LAS 1.4 file should be zero.\n", i);
+      }
+      else
+      {
+        extended_number_of_points_by_return = header->extended_number_of_points_by_return[i];
+        header->number_of_points_by_return[i] = (U32)header->extended_number_of_points_by_return[i];
+      }
+    }
     else
+    {
       extended_number_of_points_by_return = header->extended_number_of_points_by_return[i];
+    }
     out->put64bitsLE((U8*)&extended_number_of_points_by_return);
   }
   // add the compatibility VLR
@@ -185,7 +205,7 @@ BOOL LASwriterCompatibleDown::open(LASheader* header, LASwriteOpener* laswriteop
   {
     header->point_data_format = 3;
   }
-  else // 9->4 and 10->5
+  else // 9->4 and 10->5 
   {
     header->point_data_format -= 5;
   }
@@ -209,9 +229,9 @@ BOOL LASwriterCompatibleDown::open(LASheader* header, LASwriteOpener* laswriteop
           else
           {
 #ifdef _WIN32
-            throw std::runtime_error(std::string("large EVLR with user ID '%s' and record ID %d with payload size %I64d not moved to VLRs.")); //header->evlrs[i].user_id, header->evlrs[i].record_id, header->evlrs[i].record_length_after_header
+            REprintf("large EVLR with user ID '%s' and record ID %d with payload size %I64d not moved to VLRs.\n", header->evlrs[i].user_id, header->evlrs[i].record_id, header->evlrs[i].record_length_after_header);
 #else
-            throw std::runtime_error(std::string("large EVLR with user ID '%s' and record ID %d with payload size %lld not moved to VLRs.")); //header->evlrs[i].user_id, header->evlrs[i].record_id, header->evlrs[i].record_length_after_header
+            REprintf("large EVLR with user ID '%s' and record ID %d with payload size %lld not moved to VLRs.\n", header->evlrs[i].user_id, header->evlrs[i].record_id, header->evlrs[i].record_length_after_header);
 #endif
           }
         }
@@ -249,7 +269,7 @@ BOOL LASwriterCompatibleDown::open(LASheader* header, LASwriteOpener* laswriteop
           }
           else
           {
-            throw std::runtime_error(std::string("unknown LASF_Projection EVLR with record ID %d not moved to VLRs.")); //header->evlrs[i].record_id
+            REprintf("unknown LASF_Projection EVLR with record ID %d not moved to VLRs.\n", header->evlrs[i].record_id);
           }
         }
       }
@@ -333,7 +353,7 @@ BOOL LASwriterCompatibleDown::write_point(const LASpoint* point)
   scanner_channel = pointCompatibleDown.extended_scanner_channel;
   overlap_bit = (pointCompatibleDown.extended_classification_flags >> 3);
 
-  // write distilled extended attributes into extra bytes
+  // write distilled extended attributes into extra bytes 
 
   pointCompatibleDown.set_attribute(start_scan_angle, ((I16)scan_angle_remainder));
   pointCompatibleDown.set_attribute(start_extended_returns, (U8)((return_number_increment << 4) | number_of_returns_increment));
@@ -411,43 +431,43 @@ BOOL LASwriterCompatibleUp::open(LASheader* header, LASwriteOpener* laswriteopen
   const LASvlr* compatibility_vlr = header->get_vlr("lascompatible", 22204);
   if (compatibility_vlr == 0)
   {
-    throw std::runtime_error(std::string("ERROR: no compatibility VLR in header"));
+    REprintf( "ERROR: no compatibility VLR in header\n"); 
     return FALSE;
   }
   // the compatibility VLR must have the right length
   if (compatibility_vlr->record_length_after_header != (2+2+4+148))
   {
-    throw std::runtime_error(std::string("ERROR: compatibility VLR has %u instead of %u bytes in payload")); //compatibility_vlr->record_length_after_header, 2+2+4+148
+    REprintf( "ERROR: compatibility VLR has %u instead of %u bytes in payload\n", compatibility_vlr->record_length_after_header, 2+2+4+148);
     return FALSE;
   }
   I32 index_scan_angle = header->get_attribute_index("LAS 1.4 scan angle");
   if (index_scan_angle == -1)
   {
-    throw std::runtime_error(std::string("ERROR: attribute \"LAS 1.4 scan angle\" is not in EXTRA_BYTES"));
+    REprintf( "ERROR: attribute \"LAS 1.4 scan angle\" is not in EXTRA_BYTES\n");
     return FALSE;
   }
   start_scan_angle = header->get_attribute_start(index_scan_angle);
   I32 index_extended_returns = header->get_attribute_index("LAS 1.4 extended returns");
   if (index_extended_returns == -1)
   {
-    throw std::runtime_error(std::string("ERROR: attribute \"LAS 1.4 extended returns\" is not in EXTRA_BYTES"));
+    REprintf( "ERROR: attribute \"LAS 1.4 extended returns\" is not in EXTRA_BYTES\n");
     return FALSE;
   }
   start_extended_returns = header->get_attribute_start(index_extended_returns);
   I32 index_classification = header->get_attribute_index("LAS 1.4 classification");
   if (index_classification == -1)
   {
-    throw std::runtime_error(std::string("ERROR: attribute \"LAS 1.4 classification\" is not in EXTRA_BYTES"));
+    REprintf( "ERROR: attribute \"LAS 1.4 classification\" is not in EXTRA_BYTES\n");
     return FALSE;
   }
   start_classification = header->get_attribute_start(index_classification);
   I32 index_flags_and_channel = header->get_attribute_index("LAS 1.4 flags and channel");
   if (index_scan_angle == -1)
   {
-    throw std::runtime_error(std::string("ERROR: attribute \"LAS 1.4 flags and channel\" is not in EXTRA_BYTES"));
+    REprintf( "ERROR: attribute \"LAS 1.4 flags and channel\" is not in EXTRA_BYTES\n");
     return FALSE;
   }
-  start_flags_and_channel = header->get_attribute_start(index_flags_and_channel);
+  start_flags_and_channel = header->get_attribute_start(index_flags_and_channel); 
 
   this->header = header;
 
@@ -490,14 +510,14 @@ BOOL LASwriterCompatibleUp::open(LASheader* header, LASwriteOpener* laswriteopen
   in->get16bitsLE((U8*)&compatible_version);
   if (compatible_version != 3)
   {
-    throw std::runtime_error(std::string("ERROR: compatibility mode version %u not implemented")); //compatible_version
+    REprintf( "ERROR: compatibility mode version %u not implemented\n", compatible_version);
     return FALSE;
   }
   U32 unused;
   in->get32bitsLE((U8*)&unused);
   if (unused != 0)
   {
-    throw std::runtime_error(std::string("WARNING: unused is %u instead of 0")); //unused
+    REprintf( "WARNING: unused is %u instead of 0\n", unused);
   }
   in->get64bitsLE((U8*)&(header->start_of_waveform_data_packet_record));
   in->get64bitsLE((U8*)&(header->start_of_first_extended_variable_length_record));
