@@ -4,7 +4,7 @@ lazfile <- system.file("extdata", "example.laz", package="rlas")
 
 test_that("read returns good values", {
 
-  las = readlasdata(lazfile)
+  las = read.las(lazfile)
 
   expect_true(data.table::is.data.table(las))
   expect_equal(dim(las), c(30, 13))
@@ -12,7 +12,7 @@ test_that("read returns good values", {
 
 test_that("filter returns good values", {
 
-  las = readlasdata(lazfile, filter = "-keep_first")
+  las = read.las(lazfile, filter = "-keep_first")
 
   expect_true(data.table::is.data.table(las))
   expect_equal(dim(las), c(26, 13))
@@ -20,7 +20,7 @@ test_that("filter returns good values", {
 
 test_that("colunm selection works", {
 
-  las = readlasdata(lazfile, t = F, i = F)
+  las = read.las(lazfile, select = "xyzrndecaupRGBN0")
 
   expect_true(data.table::is.data.table(las))
   expect_equal(dim(las), c(30, 11))
@@ -28,7 +28,7 @@ test_that("colunm selection works", {
   expect_true(!"Intensity" %in% names(las))
   expect_true("Classification" %in% names(las))
 
-  las = readlasdata(lazfile, p = F, c = F)
+  las = read.las(lazfile, select = "xyztirndeauRGBN0")
 
   expect_true(data.table::is.data.table(las))
   expect_equal(dim(las), c(30, 11))
@@ -37,12 +37,26 @@ test_that("colunm selection works", {
   expect_true(!"Classification" %in% names(las))
 })
 
+test_that("colunm unselection works", {
+
+  las1 = read.las(lazfile, select = "xyzrndecaupRGBN0")
+  las2 = read.las(lazfile, select = "* -i -t")
+
+  expect_equal(las1, las2)
+
+  las1 = read.las(lazfile, select = "xyztirndeauRGBN0")
+  las2 = read.las(lazfile, select = "* -c -p")
+
+  expect_equal(las1, las2)
+})
+
+
 test_that("streaming works", {
   ofile = paste0(tempfile(), ".las")
 
-  las1 = readlasdata(lazfile, filter = "-keep_first")
-  rlas:::streamlasdata(lazfile, ofile, "-keep_first")
-  las2 = readlasdata(ofile)
+  las1 = read.las(lazfile, filter = "-keep_first")
+  rlas:::stream.las(lazfile, ofile, filter = "-keep_first")
+  las2 = read.las(ofile)
 
   expect_equal(las1, las2)
 })
@@ -51,18 +65,17 @@ test_that("streaming works with extra bytes", {
   lazfile <- system.file("extdata", "extra_byte.laz", package="rlas")
   ofile = paste0(tempfile(), ".las")
 
-  las1 = readlasdata(lazfile, filter = "-keep_first")
-  rlas:::streamlasdata(lazfile, ofile, "-keep_first")
-  las2 = readlasdata(ofile)
+  las1 = read.las(lazfile, filter = "-keep_first")
+  rlas:::stream.las(lazfile, ofile, filter = "-keep_first")
+  las2 = read.las(ofile)
 
   expect_equal(las1, las2)
 })
 
 test_that("streaming reads if no ofile", {
   ofile = ""
-
-  las1 = readlasdata(lazfile, filter = "-keep_first")
-  las2 = rlas:::streamlasdata(lazfile, ofile, "-keep_first")
+  las1 = read.las(lazfile, filter = "-keep_first")
+  las2 = rlas:::stream.las(lazfile, ofile, filter = "-keep_first")
 
   expect_equal(las1, las2)
 })
@@ -73,7 +86,7 @@ test_that("read in poly works", {
   xpoly = c(8, 10, 11, 7, 8) + 339000
   ypoly = c(0, 0, 1, 1, 0) + 5248000
 
-  las = rlas:::streamlasdata_inpoly(lazfile, xpoly, ypoly, ofile, filter = "")
+  las = rlas:::stream.las_inpoly(lazfile, xpoly, ypoly, ofile, filter = "")
 
   expect_equal(dim(las), c(14, 13))
 })
@@ -85,7 +98,7 @@ test_that("read in poly works with filter and select", {
   xpoly = c(8, 10, 11, 7, 8) + 339000
   ypoly = c(0, 0, 1, 1, 0) + 5248000
 
-  las = rlas:::streamlasdata_inpoly(lazfile, xpoly, ypoly, ofile, filter = "-drop_z_above 977.5", i = FALSE, t = FALSE)
+  las = rlas:::stream.las_inpoly(lazfile, xpoly, ypoly, ofile, select = "* -t -i", filter = "-drop_z_above 977.5")
 
   expect_equal(dim(las), c(8, 11))
 })
@@ -93,7 +106,7 @@ test_that("read in poly works with filter and select", {
 test_that("extra byte selection works", {
   lazfile <- system.file("extdata", "extra_byte.laz", package="rlas")
 
-  las = readlasdata(lazfile, eb = 0)
+  las = read.las(lazfile)
 
   expect_true("Pulse width" %in% names(las))
   expect_true("Amplitude" %in% names(las))
@@ -101,7 +114,7 @@ test_that("extra byte selection works", {
 
   pw1 = las$`Pulse width`
 
-  las = readlasdata(lazfile, eb = c(2,5))
+  las = read.las(lazfile, select = "xyztirndecaupRGBN25")
 
   expect_true("Pulse width" %in% names(las))
   expect_false("Amplitude" %in% names(las))
@@ -111,13 +124,13 @@ test_that("extra byte selection works", {
 
   expect_equal(pw1, pw2)
 
-  las = readlasdata(lazfile, eb = numeric(0))
+  las = read.las(lazfile, select = "xyztirndecaupRGBN")
 
   expect_false("Pulse width" %in% names(las))
   expect_false("Amplitude" %in% names(las))
   expect_equal(ncol(las), 13)
 
-  las = readlasdata(lazfile, eb = NULL)
+  las = read.las(lazfile, select = "* -0")
 
   expect_false("Pulse width" %in% names(las))
   expect_false("Amplitude" %in% names(las))
